"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isIterable = isIterable;
exports.isAsyncIterable = isAsyncIterable;
exports.isIteratorLike = isIteratorLike;
exports.isIterableIterator = isIterableIterator;
exports.isAsyncIterableIterator = isAsyncIterableIterator;
exports.isGenerator = isGenerator;
exports.isAsyncGenerator = isAsyncGenerator;

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

if (!Symbol.asyncIterator) {
  Symbol.asyncIterator = Symbol("Symbol.asyncIterator");
}
/**
 * Checks if the given object is an Iterable (implemented `@@iterator`).
 * @returns {obj is Iterable<any>}
 */


function isIterable(obj) {
  return obj !== null && obj !== undefined && typeof obj[Symbol.iterator] === "function";
}
/**
 * Checks if the given object is an AsyncIterable (implemented `@@asyncIterator`).
 * @returns {obj is AsyncIterable<any>}
 */


function isAsyncIterable(obj) {
  return obj !== null && obj !== undefined && typeof obj[Symbol.asyncIterator] === "function";
}
/**
 * Checks if the given object is an IteratorLike (implemented `next`).
 * @returns {obj is { next: Function }}
 */


function isIteratorLike(obj) {
  // An iterable object has a 'next' method, however including a 'next' method
  // doesn't ensure the object is an iterator, it is only iterator-like.
  return _typeof(obj) === "object" && obj !== null && typeof obj.next === "function";
}
/**
 * Checks if the given object is an IterableIterator (implemented both
 * `@@iterator` and `next`).
 */


function isIterableIterator(obj) {
  return isIteratorLike(obj) && typeof obj[Symbol.iterator] === "function";
}
/**
 * Checks if the given object is an AsyncIterableIterator (implemented
 * both `@@asyncIterator` and `next`).
 * @returns {obj is AsyncIterableIterator<any>}
 */


function isAsyncIterableIterator(obj) {
  return isIteratorLike(obj) && typeof obj[Symbol.asyncIterator] === "function";
}
/**
 * Checks if the given object is a Generator.
 * @returns {obj is Generator}
 */


function isGenerator(obj) {
  return isIterableIterator(obj) && hasGeneratorSpecials(obj);
}
/**
 * Checks if the given object is an AsyncGenerator.
 * @returns {obj is AsyncGenerator}
 */


function isAsyncGenerator(obj) {
  return isAsyncIterableIterator(obj) && hasGeneratorSpecials(obj);
}

function hasGeneratorSpecials(obj) {
  return typeof obj["return"] === "function" && typeof obj["throw"] === "function";
}
